unit AppMainU;

{
 Real use of TRPCBuffer is shown in internet communications
 and it is ideal for multitier databases as it carries
 data in its true form, regardless the database server
 and network type.
 Major advantage is speed and easy navigation through it.

 This demo shows a use of TRPCBuffer as simple
 data transfer to/from external DLL which communicates
 with database and sends requested data to the calling
 application.
 "Simple" in this case donesn't assume that you are beginner
 in Delphi, but usage of TRPCBuffer is simple and it is
 demonstrated on simple way.
 Alas, DLL communication is deliberately chosen in this
 example as it is more complex then simple sending of data
 through the Socket and requires more caution and
 knowledge in dealing with memory.

  Green House multimedia, 2001.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, BufferDef;

type
  TForm1 = class(TForm)
    ComboBox1: TComboBox;
    Memo1: TMemo;
    Button1: TButton;
    Label1: TLabel;
    procedure FormShow(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure ComboBox1Change(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

{
 Decalaration of only exported procedure in DLL.
 As this DLL is ment to be used by all programming
 languages, I deliberately pass PChar instead of
 TRPCBuffer, as in which case, it could be used only
 by Delphi.
 }
procedure SendCommand(buf: PChar; len: Integer; OutPut: PChar); stdcall;
external 'DemoDll.dll';

var
  Form1: TForm1;
  Command, Buffer: TRPCBuffer;

implementation

{$R *.DFM}

procedure TForm1.FormShow(Sender: TObject);
var
  i: integer;
  p1, p2: PChar;
begin
{ Order DLL to load and pass to ComboBox list of countries
 from Database }
  Command := TRPCBuffer.create;
  Buffer := TRPCBuffer.create;
  try
   //Put command "LOAD" into the first field into
   //the TRPCBuffer
    Command.WriteFields(true, ['LOAD']);
    //Get Data from TRPCBuffer. Data is PChar type
    //and it is held by DataBuffer property of TRPCBuffer
    p1 := command.DataBuffer;


    GetMem(p2, 2048);
    {
     As I made very simple procedure that does all the work,
     and not to make things more complicated, as making
     3 passes inside of DLL (procedures/functions) that
     will know the size of return buffer, I "ASSUMED"
     size of the p2 PChar to 2048 bytes.
     There are lots of solutions to this, so pick up
     one that suits you the best.
    }


    {
    Pass command to DLL, with size of the PChar
    and finally get the result buffer into
    p2 PChar which I allocated in previous command
    }
    SendCommand(p1, command.DataBufferSize, p2);
    if p2 = nil then
    begin
      showmessage('ERROR: Buffer is empty!');
      exit;
    end;
    buffer.Rewrite; //Initialze buffer

    // Add result to the buffer which is TRPCBuffer
    buffer.DataBuffer := p2;
    //Set the length and data count
    Buffer.DataBufferCount := Length(p2);
    //Move to the first record of the buffer
    buffer.first;

    //Loop through buffer and load combobox
    for i := 0 to buffer.RecordCount - 1 do
    begin
      ComboBox1.items.add(Buffer.fields[0]);
      buffer.next;
    end;
  finally
    FreeAndNil(Command);
    FreeMem(p2); //just to be sure. It is not necessary
    Buffer.free;
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  i: integer;
  s: string;
  p1, p2: PChar;
begin
  Command := TRPCBuffer.create;
  Buffer := TRPCBuffer.create;
  try
   //Write command in new record, at field 1
    Command.WriteFields(true, ['GET']);
   // Add new field in same record and add a Country name
    command.WriteFields(false, [ComboBox1.text]);
    p1 := command.DataBuffer;
    //Assumed value, read comment in OnShow
    GetMem(p2, 4092);
    //Pass command and size of it to DLL and
    //load the result into p2
    SendCommand(p1, command.DataBufferSize, p2);
    if p2 = nil then
    begin
      showmessage('ERROR: Buffer is empty!');
      exit;
    end;
    buffer.Rewrite; //Initialize buffer
    buffer.DataBuffer := p2; //Copy data to RPCBuffer
    Buffer.DataBufferCount := Length(p2); //Set length
    buffer.first;
    memo1.lines.clear;
    //Loop through result buffer and load Memo
    for i := 0 to buffer.RecordCount - 1 do
    begin
      case i of
        0: s := 'Country: ';
        1: s := 'Capital: ';
        2: s := 'Continent: ';
        3: s := 'Area: ';
        4: s := 'Population: ';
      end;
      Memo1.Lines.add(s + Buffer.fields[0]);
      buffer.next;
    end;
  finally
    FreeAndNil(Command);
    Buffer.free;
  end;
end;

procedure TForm1.ComboBox1Change(Sender: TObject);
begin
  Button1.enabled := true;
end;

end.

